/**
 * Fractal Binding Factory.
 * Copyright (C) 2007-2008 INRIA, SARDES

 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * Contact: Valerio.Schiavoni@gmail.com
 * 
 * Author: Valerio Schiavoni
 * 
 */
package org.objectweb.fractal.bf.connectors.rmi;

import java.rmi.NotBoundException;
import java.rmi.RemoteException;
import java.rmi.registry.LocateRegistry;
import java.rmi.registry.Registry;
import java.util.logging.Logger;

import org.objectweb.fractal.bf.AbstractRemoteServiceCaller;
import org.objectweb.fractal.bf.BindingFactoryException;

/**
 * Dispatch calls to the RMI registry and return the result of the call to the
 * stub component this handler will be part of.
 */
public class RmiServiceCaller extends AbstractRemoteServiceCaller {

	Logger log = Logger.getLogger(RmiServiceCaller.class.getCanonicalName());

	/**
	 * the name of the service this invocation handler will dispatch calls to.
	 */
	private final String name;
	/**
	 * The host address of the RMI registry.
	 */
	private final String hostAddress;
	/**
	 * The port of the RMI regitry.
	 */
	private final String registryPort;
	/**
	 * The RMI registry.
	 */
	private Registry registry;

	/**
	 * @param params
	 */
	public RmiServiceCaller(String name, String hostAddress, String registryPort) {
		this.name = name;
		this.hostAddress = hostAddress;
		this.registryPort = registryPort;
	}

	public RmiServiceCaller(RmiBindHints bindHints)
			throws BindingFactoryException {
		this(bindHints.getServiceName(), bindHints.getHost(), bindHints
				.getPort());

		try {
			this.registry = LocateRegistry.getRegistry(hostAddress, Integer
					.valueOf(registryPort).intValue());
			this.reference = registry.lookup(this.name);
		} catch (NumberFormatException e) {
			throw new BindingFactoryException(e);
		} catch (RemoteException e) {
			throw new BindingFactoryException(e);

		} catch (NotBoundException e) {
			throw new BindingFactoryException(e);
		}
		if (registry == null) {
			throw new BindingFactoryException("Cannot locate JavaRMi registry");
		}

	}

	@Override
	protected void beforeInvokeHook() {
		if (this.reference == null) {
			try {
				log.finest("Recovering service from RMI registry with name: "
						+ this.name);
				this.reference = registry.lookup(this.name);

			} catch (Exception e) {
				throw new RmiConnectorException(e);
			}
		}

	}

}
